@@ -410,6 +410,85 @@ def create_dynamic_model(
 
         validate_json_schema(schema)
 
+        # ------------------------------------------------------------------
+        # Ensure every subschema can resolve local references ("#/definitions/…")
+        # by copying the root-level `definitions` block into any nested object
+        # that lacks its own. This is purely an in-memory fix; the original file
+        # on disk is not modified.
+        # ------------------------------------------------------------------
+
+        root_definitions = schema.get("definitions")
+
+        if root_definitions:
+            seen_ids: set[int] = set()
+
+            def _inject_defs(
+                sub: Any,
+            ) -> None:  # noqa: ANN401 – recursive helper
+                """Recursively add root definitions to nested dicts."""
+
+                if isinstance(sub, dict):
+                    obj_id = id(sub)
+                    if obj_id in seen_ids:
+                        return  # Prevent infinite loops
+                    seen_ids.add(obj_id)
+
+                    # Only inject if this dict isn't itself the shared definitions
+                    if "definitions" not in sub:
+                        sub["definitions"] = root_definitions
+
+                    for key, val in sub.items():
+                        # Skip traversing into the definitions block we just attached
+                        if key == "definitions":
+                            continue
+                        _inject_defs(val)
+
+                elif isinstance(sub, list):
+                    for item in sub:
+                        _inject_defs(item)
+
+            # Start recursion on key fields likely to contain subschemas
+            for key in (
+                "properties",
+                "items",
+                "additionalProperties",
+                "allOf",
+                "anyOf",
+                "oneOf",
+                "then",
+                "else",
+                "if",
+            ):
+                if key in schema:
+                    _inject_defs(schema[key])
+
+            # ------------------------------------------------------------------
+            # OpenAI Responses API does not yet accept $ref. Replace any local
+            # reference that points into the root-level definitions with the
+            # referenced schema in-place so the final schema is fully expanded.
+            # ------------------------------------------------------------------
+
+            def _expand_refs(node: Any) -> Any:  # noqa: ANN401
+                if isinstance(node, dict):
+                    if "$ref" in node:
+                        ref_val: str = node["$ref"]
+                        if ref_val.startswith("#/definitions/"):
+                            ref_key = ref_val.split("/", 2)[-1]
+                            if ref_key in root_definitions:
+                                return _expand_refs(root_definitions[ref_key])
+                        # Unknown refs fall through unchanged (will error later)
+                    # Recurse into values
+                    return {
+                        k: _expand_refs(v)
+                        for k, v in node.items()
+                        if k != "definitions"
+                    }
+                if isinstance(node, list):
+                    return [_expand_refs(i) for i in node]
+                return node
+
+            schema = _expand_refs(schema)
+
         # Handle top-level array schemas
         if schema.get("type") == "array":
             items_schema = schema.get("items", {})
