@@ -112,22 +112,35 @@ def parse_json_with_duplication_handling(content: str) -> Any:
 
     Raises:
         json.JSONDecodeError: If JSON parsing fails completely
+        JSONSizeError: If JSON content exceeds size limits
+        JSONDepthError: If JSON content exceeds depth limits
+        JSONComplexityError: If JSON content exceeds complexity limits
 
     References:
         - https://community.openai.com/t/gpt-4o-structured-outputs-json-extra-data/884253
         - https://community.openai.com/t/gpt-4o-structured-outputs-duplicate-json/883156
     """
+    from .json_limits import (
+        JSONComplexityError,
+        JSONDepthError,
+        JSONSizeError,
+        parse_json_secure,
+    )
+
     config = get_config()
     strategy = config.json_parsing_strategy
 
     if strategy == "strict":
-        # Strict mode: fail on any malformed JSON
-        return json.loads(content)
+        # Strict mode: fail on any malformed JSON, with security limits
+        return parse_json_secure(content)
 
-    # Robust mode: handle duplication bugs
+    # Robust mode: handle duplication bugs with security limits
     try:
-        # Try normal parsing first
-        return json.loads(content)
+        # Try secure parsing first
+        return parse_json_secure(content)
+    except (JSONSizeError, JSONDepthError, JSONComplexityError):
+        # Security limit exceeded - don't attempt recovery
+        raise
     except json.JSONDecodeError as e:
         # Check if this is the specific "Extra data" error indicating duplication
         if "Extra data" in str(e) and hasattr(e, "pos"):
@@ -141,7 +154,7 @@ def parse_json_with_duplication_handling(content: str) -> Any:
             # Split at the error position and try to parse the first JSON object
             try:
                 first_json = content[: e.pos].strip()
-                result = json.loads(first_json)
+                result = parse_json_secure(first_json)
                 logger.info(
                     f"Successfully recovered from JSON duplication bug by parsing first {e.pos} characters. "
                     f"Discarded {len(content) - e.pos} duplicate characters."
@@ -152,6 +165,9 @@ def parse_json_with_duplication_handling(content: str) -> Any:
                     f"Failed to recover from JSON duplication - first part is also malformed: {inner_e}"
                 )
                 raise e  # Re-raise original error
+            except (JSONSizeError, JSONDepthError, JSONComplexityError):
+                # Security limit exceeded even in first part - don't attempt further recovery
+                raise
         else:
             # Different JSON error, re-raise as-is
             raise
@@ -704,16 +720,33 @@ async def create_structured_output(
                     "tool_choice applied: %s", api_params["tool_choice"]
                 )
 
-        # Log the API request details
+        # Log the API request details with credential sanitization
+        from .security.credential_sanitizer import CredentialSanitizer
+
         logger.debug("Making OpenAI Responses API request with:")
         logger.debug("Model: %s", model)
-        logger.debug("Message content: %s", message_content)
-        logger.debug("Parameters: %s", json.dumps(api_kwargs, indent=2))
+        logger.debug(
+            "Message content: %s",
+            CredentialSanitizer.sanitize_string(message_content),
+        )
+        logger.debug(
+            "Parameters: %s",
+            json.dumps(
+                CredentialSanitizer.sanitize_dict(api_kwargs), indent=2
+            ),
+        )
         logger.debug("Schema: %s", json.dumps(strict_schema, indent=2))
-        logger.debug("Tools being passed to API: %s", tools)
+        logger.debug(
+            "Tools being passed to API: %s",
+            CredentialSanitizer.sanitize_for_logging(tools),
+        )
         logger.debug(
             "Complete API params: %s",
-            json.dumps(api_params, indent=2, default=str),
+            json.dumps(
+                CredentialSanitizer.sanitize_dict(api_params),
+                indent=2,
+                default=str,
+            ),
         )
 
         # Use the Responses API
@@ -866,20 +899,24 @@ async def create_structured_output(
             or "maximum context length" in error_msg
         ):
             raise CLIError(
-                f"Context length exceeded: {str(e)}",
+                f"Context length exceeded: {CredentialSanitizer.sanitize_exception(e)}",
                 exit_code=ExitCode.API_ERROR,
             )
         elif "rate_limit" in error_msg or "429" in str(e):
             raise CLIError(
-                f"Rate limit exceeded: {str(e)}", exit_code=ExitCode.API_ERROR
+                f"Rate limit exceeded: {CredentialSanitizer.sanitize_exception(e)}",
+                exit_code=ExitCode.API_ERROR,
             )
         elif "invalid_api_key" in error_msg:
             raise CLIError(
-                f"Invalid API key: {str(e)}", exit_code=ExitCode.API_ERROR
+                f"Invalid API key: {CredentialSanitizer.sanitize_exception(e)}",
+                exit_code=ExitCode.API_ERROR,
             )
         else:
-            logger.error(f"Unmapped API error: {e}")
-            raise APIResponseError(str(e))
+            logger.error(
+                f"Unmapped API error: {CredentialSanitizer.sanitize_exception(e)}"
+            )
+            raise APIResponseError(CredentialSanitizer.sanitize_exception(e))
 
 
 # Note: validation functions are defined in cli.py to avoid circular imports
@@ -1893,16 +1930,6 @@ async def run_cli_async(args: CLIParams) -> ExitCode:
         CLIError: For errors during CLI operations.
     """
     try:
-        # 0. Configure Debug Logging
-        from .template_debug import configure_debug_logging
-
-        configure_debug_logging(
-            verbose=bool(args.get("verbose", False)),
-            debug=bool(args.get("debug", False)),
-        )
-
-        # 0a. Help Debug Request is now handled by callback in click_options.py
-
         # 0. Configure Progress Reporting
         configure_progress_reporter(
             verbose=args.get("verbose", False),
