@@ -51,21 +51,22 @@ Known Limitations:
 """
 
 import os
-import re
 import unicodedata
 from pathlib import Path
 from typing import Union
 
+# Import safe regex patterns
+from ..safe_regex import (
+    SAFE_BACKSLASH_PATTERN,
+    SAFE_MULTIPLE_SLASH_PATTERN,
+    SAFE_UNICODE_PATTERN,
+)
 from .errors import PathSecurityError, SecurityErrorReasons
 
-# Patterns for path normalization and validation
-_UNICODE_SAFETY_PATTERN = re.compile(
-    r"[\u0000-\u001F\u007F-\u009F\u2028-\u2029\u0085]"  # Control chars and line separators
-    r"|(?:^|/)\.\.(?:/|$)"  # Directory traversal attempts (only ".." as a path component)
-    r"|[\u2024\u2025\uFE52\u2024\u2025\u2026\uFE19\uFE30\uFE52\uFF0E\uFF61]"  # Alternative dots and separators
-)
-_BACKSLASH_PATTERN = re.compile(r"\\")
-_MULTIPLE_SLASH_PATTERN = re.compile(r"/+")
+# Patterns for path normalization and validation (ReDoS-safe)
+_UNICODE_SAFETY_PATTERN = SAFE_UNICODE_PATTERN
+_BACKSLASH_PATTERN = SAFE_BACKSLASH_PATTERN
+_MULTIPLE_SLASH_PATTERN = SAFE_MULTIPLE_SLASH_PATTERN
 
 
 def normalize_path(
