@@ -254,12 +254,16 @@ def to_json(obj: Any) -> str:
 
     # Lazy import to avoid heavy deps at module import time
     try:
-        from .attachment_template_bridge import LazyFileContent
+        from .file_info import FileInfo
     except Exception:  # pragma: no cover
-        LazyFileContent = None  # type: ignore
+        FileInfo = None  # type: ignore
 
-    # 1️⃣  Unwrap LazyFileContent → underlying text
-    if LazyFileContent is not None and isinstance(obj, LazyFileContent):
+    # 1️⃣  Unwrap FileInfo with lazy loading → underlying text
+    if (
+        FileInfo is not None
+        and isinstance(obj, FileInfo)
+        and getattr(obj, "_FileInfo__lazy_loading", False)
+    ):
         obj = str(obj)  # forces lazy load, returns text
 
     # 2️⃣  Attempt normal serialisation first
@@ -273,6 +277,8 @@ def to_json(obj: Any) -> str:
             return f"<unserialisable:{type(value).__name__}>"
 
     try:
+        # Use standard json.dumps for serialization (not parsing)
+        # The security limits are for parsing JSON, not generating it
         return json.dumps(obj, default=_default_fallback)
     except Exception as e:  # pragma: no cover – shouldn't happen now
         logger.debug("to_json serialisation fallback hit: %s", e)
