@@ -2,7 +2,6 @@
 
 import json
 import logging
-import re
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
@@ -307,13 +306,29 @@ def validate_task_template(
     else:
         template_content = task  # type: ignore  # We know task is str here due to the checks above
 
+    # Sanitize template content for security
+    from .template_sanitizer import TemplateSanitizer
+
+    try:
+        template_content = TemplateSanitizer.validate_template_content(
+            template_content
+        )
+    except ValueError as e:
+        raise TaskTemplateSyntaxError(
+            f"Template security validation failed: {e}",
+            context={
+                "template_file": task_file,
+                "template_preview": template_content[:200],
+            },
+        )
+
     try:
         env = jinja2.Environment(undefined=jinja2.StrictUndefined)
         env.parse(template_content)
         return template_content
     except jinja2.TemplateSyntaxError as e:
         raise TaskTemplateSyntaxError(
-            f"Invalid Jinja2 template syntax: {e.message}",
+            f"Invalid template syntax: {e.message}",
             context={
                 "line": e.lineno,
                 "template_file": task_file,
@@ -499,7 +514,23 @@ def collect_json_variables(args: CLIParams) -> Dict[str, Any]:
                             f"Invalid JSON variable mapping format: {mapping}. Expected name=json"
                         )
                     try:
-                        value = json.loads(json_str)
+                        from .json_limits import (
+                            JSONComplexityError,
+                            JSONDepthError,
+                            JSONSizeError,
+                            parse_json_secure,
+                        )
+
+                        value = parse_json_secure(json_str)
+                    except (
+                        JSONSizeError,
+                        JSONDepthError,
+                        JSONComplexityError,
+                    ) as e:
+                        raise InvalidJSONError(
+                            f"JSON value for variable '{name}' exceeds security limits: {e}",
+                            context={"variable_name": name},
+                        ) from e
                     except json.JSONDecodeError as e:
                         raise InvalidJSONError(
                             f"Invalid JSON value for variable '{name}': {json_str}",
@@ -673,35 +704,6 @@ def _build_tool_context(
     return context
 
 
-def _generate_template_variable_name(file_path: str) -> str:
-    """Generate a template variable name from a file path.
-
-    Converts filename to a valid template variable name by:
-    1. Taking the full filename (with extension)
-    2. Replacing dots and other special characters with underscores
-    3. Ensuring it starts with a letter or underscore
-
-    Examples:
-        data.csv -> data_csv
-        data.json -> data_json
-        my-file.txt -> my_file_txt
-        123data.xml -> _123data_xml
-
-    Args:
-        file_path: Path to the file
-
-    Returns:
-        Valid template variable name
-    """
-    filename = Path(file_path).name
-    # Replace special characters with underscores
-    var_name = re.sub(r"[^a-zA-Z0-9_]", "_", filename)
-    # Ensure it starts with letter or underscore
-    if var_name and var_name[0].isdigit():
-        var_name = "_" + var_name
-    return var_name
-
-
 async def create_template_context_from_routing(
     args: CLIParams,
     security_manager: SecurityManager,
